import launch
from launch_ros.actions import Node


def generate_launch_description():

    use_components_config = launch.substitutions.LaunchConfiguration('use_components', default=False)

    ### Manual composition of the image processing components
    lane_detection_pipeline = Node(
        package='dataspeed_dbw_gazebo_demo',
        namespace='vehicle/front_camera',
        executable='dataspeed_dbw_gazebo_demo_pipeline',
        parameters=[{
            'use_sim_time': True
        }],
        condition=launch.conditions.IfCondition(use_components_config)
    )

    ### Separate processes for each image processing node
    lane_detection_node = Node(
        package='dataspeed_dbw_gazebo_demo',
        namespace='vehicle/front_camera',
        executable='dataspeed_dbw_gazebo_demo_lane_detection_node',
        parameters=[{
            'use_sim_time': True
        }],
        name='lane_detection',
        condition=launch.conditions.UnlessCondition(use_components_config)
    )

    lane_fit_node = Node(
        package='dataspeed_dbw_gazebo_demo',
        namespace='vehicle/front_camera',
        executable='dataspeed_dbw_gazebo_demo_lane_fit_node',
        parameters=[{
            'use_sim_time': True
        }],
        name='lane_fit',
        condition=launch.conditions.UnlessCondition(use_components_config)
    )

    lane_planner_node = Node(
        package='dataspeed_dbw_gazebo_demo',
        namespace='vehicle/front_camera',
        executable='dataspeed_dbw_gazebo_demo_lane_planner_node',
        parameters=[{
            'use_sim_time': True
        }],
        name='lane_planning',
        condition=launch.conditions.UnlessCondition(use_components_config)
    )


    ### Standalone path following node
    path_following_node = Node(
        package='dataspeed_dbw_gazebo_demo',
        namespace='vehicle/front_camera',
        executable='dataspeed_dbw_gazebo_demo_path_following_node',
        parameters=[{
            'use_sim_time': True
        }],
        name='path_following',
    )

    enable_dbw = launch.actions.ExecuteProcess(
        cmd=['ros2', 'topic', 'pub', '/vehicle/enable', 'std_msgs/Empty'],
        output='screen'
    )

    return launch.LaunchDescription([lane_detection_pipeline, lane_detection_node, lane_fit_node, lane_planner_node, path_following_node, enable_dbw])
